/////////////////////////////////////////////////////////////////////////
///@system  开放平台API接口
///@company 浙江核新同花顺网络信息股份有限公司
///@file    hx_trader_api_struct.h
///@brief   定义交易类相关数据结构
/////////////////////////////////////////////////////////////////////////

#ifndef _HX_TRADER_API_STRUCT_H_
#define _HX_TRADER_API_STRUCT_H_

#include <string.h>
#include "hx_api_struct_common.h"
#pragma pack(8)

/////////////////////////////////////////////////////////////////////////
///新建订单请求
/////////////////////////////////////////////////////////////////////////
struct HXOrderInsertInfo
{
    ///系统订单ID 用户无需填写, 在HX_OEMS系统中唯一
    uint64_t                order_no;
    ///本地订单号 调用方传递(可以不填), 委托确认/成交回报中数据原样返回
    char                    local_order_no[HX_LOCAL_ORDER_LEN];
    ///证券代码 客户端请求不带空格,以'\0'结尾
    char                    symbol[HX_SYMBOL_LEN];
    ///交易市场
    HX_EXCHANGE_TYPE        exchange;
    ///价格
    double                  price;
    ///数量 (股票单位为股, 债券单位为张, 基金单位为份)
    int64_t                 quantity;
    ///报单价格类型
    HX_PRICE_TYPE           price_type;
    ///买卖方向
    HX_ACTION_TYPE          action;    
    ///业务类型    
    HX_BUSINESS_TYPE        business_type;    
    
    HXOrderInsertInfo()
     : order_no(0)
     , exchange(HX_EXCHANGE_UNKNOWN)
     , price(0.0)
     , quantity(0)
     , price_type(HX_PRICE_TYPE_UNKNOWN)
     , action(HX_ACTION_TYPE_UNKNOWN)
     , business_type(HX_BUSINESS_TYPE_UNKNOWN)
	{
		memset(local_order_no, 0, sizeof(local_order_no));
		memset(symbol, 0, sizeof(symbol));
	}
};

/////////////////////////////////////////////////////////////////////////
///报单响应结构体
/////////////////////////////////////////////////////////////////////////
struct HXOrderRspInfo
{
    ///系统订单ID 用户无需填写, 在HX_OEMS系统中唯一
    uint64_t                order_no;
    ///本地订单号 调用方传递(可以不填), 委托确认/成交回报中数据原样返回
    char                    local_order_no[HX_LOCAL_ORDER_LEN];
    ///证券代码 客户端请求不带空格,以'\0'结尾
    char                    symbol[HX_SYMBOL_LEN];
    ///交易市场
    HX_EXCHANGE_TYPE        exchange;
    ///价格
    double                  price;
    ///数量 (股票单位为股, 债券单位为张, 基金单位为份)
    int64_t                 quantity;
    ///报单价格
    HX_PRICE_TYPE           price_type;
    ///买卖方向
    HX_ACTION_TYPE          action;    
    ///业务类型
    HX_BUSINESS_TYPE        business_type;
    ///今成交数量 为此订单累计成交数量
    int64_t                 quantity_traded;
    ///剩余数量 当撤单成功时表示撤单数量
    int64_t                 quantity_left;
    ///委托时间 格式为YYYYMMDDHHMMSSsss
    int64_t                 order_time;
    ///撤销时间 格式为YYYYMMDDHHMMSSsss
    int64_t                 cancel_time;
    ///冻结资金    
    double                  frozen_margin;
    ///成交金额 为此订单的成交总金额
    double                  trade_amount;
    ///报单状态 订单响应中没有部分成交状态的推送, 在查询订单结果中, 会有部分成交状态
    HX_ORDER_STATUS_TYPE    order_status;
    ///报单类型
    HX_ORDER_TYPE           order_type;
    ///回报信息
    HX_MESSAGE              order_message;
	///原委托号（对于撤单有效） 
	uint64_t                order_no_source;
    
	HXOrderRspInfo()
		: order_no(0)
		, exchange(HX_EXCHANGE_UNKNOWN)
        , price(0.0)
        , quantity(0)
        , price_type(HX_PRICE_TYPE_UNKNOWN)
		, action(HX_ACTION_TYPE_UNKNOWN)
		, business_type(HX_BUSINESS_TYPE_UNKNOWN)
		, quantity_traded(0)
		, quantity_left(0)
		, order_time(0)
		, cancel_time(0)
		, frozen_margin(0.0)
		, trade_amount(0.0)
		, order_status(HX_ORDER_STATUS_TYPE_UNKNOWN)
		, order_type(HX_ORDER_TYPE_UNKNOWN)
		, order_no_source(0)
		, order_message()
	{
		memset(local_order_no, 0, sizeof(local_order_no));
		memset(symbol, 0, sizeof(symbol));
	}
};

/////////////////////////////////////////////////////////////////////////
///撤单失败响应消息
/////////////////////////////////////////////////////////////////////////
struct HXOrderCancelRspInfo
{
    ///撤单 HXID
    uint64_t                order_no;
    ///原始订单 HXID
    uint64_t                order_no_source;
	HXOrderCancelRspInfo()
		: order_no(0)
		, order_no_source(0)
	{
	}
};

/////////////////////////////////////////////////////////////////////////
///报单成交结构体
/////////////////////////////////////////////////////////////////////////
struct HXTradeReportInfo
{
    ///系统订单ID 用户无需填写, 在HX_OEMS系统中唯一
    uint64_t                order_no;
    ///本地订单号 调用方传递(可以不填), 委托确认/成交回报中数据原样返回
    char                    local_order_no[HX_LOCAL_ORDER_LEN];
    ///证券代码 客户端请求不带空格,以'\0'结尾
    char                    symbol[HX_SYMBOL_LEN];
    ///交易市场
    HX_EXCHANGE_TYPE        exchange;
    ///成交编号，深交所唯一，上交所每笔交易唯一，当发现2笔成交回报拥有相同的trade_no，则可以认为此笔交易自成交
    char                    trade_no[HX_TRADE_NO_LEN];
    ///价格，此次成交的价格
    double                  price;
    ///数量，此次成交的数量，不是累计数量
    int64_t                 quantity;
    ///成交时间，格式为YYYYMMDDHHMMSSsss
    int64_t                 trade_time;
    ///成交金额，此次成交的总金额 = price*quantity
    double                  trade_amount;
    ///成交序号 --回报记录号，每个交易所唯一,report_index+market字段可以组成唯一标识表示成交回报
    uint64_t                report_index;
    ///买卖方向
    HX_ACTION_TYPE          action;    
    ///业务类型
    HX_BUSINESS_TYPE        business_type;

	HXTradeReportInfo()
		: order_no(0)
		, exchange(HX_EXCHANGE_UNKNOWN)
		, price(0.0)
		, quantity(0)
		, trade_time(0)
		, trade_amount(0.0)
		, report_index(0)
		, action(HX_ACTION_TYPE_UNKNOWN)
		, business_type(HX_BUSINESS_TYPE_UNKNOWN)
	{
		memset(local_order_no, 0, sizeof(local_order_no));
		memset(symbol, 0, sizeof(symbol));
		memset(trade_no, 0, sizeof(trade_no));
	}
};

/////////////////////////////////////////////////////////////////////////
///报单查询
/////////////////////////////////////////////////////////////////////////
///报单查询请求-条件查询
struct HXQueryOrderReq
{
    ///系统订单ID 用户无需填写, 在HX_OEMS系统中唯一
    uint64_t                order_no;
    ///证券代码，可以为空，如果为空，则默认查询时间段内的所有成交回报
    char                    symbol[HX_SYMBOL_LEN];
    ///格式为YYYYMMDDHHMMSSsss，为0则默认当前交易日0点
    int64_t                 begin_time;
    ///格式为YYYYMMDDHHMMSSsss，为0则默认当前时间
    int64_t                 end_time;  

	HXQueryOrderReq()
		: order_no(0)
		, begin_time(0)
		, end_time(0)
	{
		memset(symbol, 0, sizeof(symbol));
	}
};

///报单查询响应结构体
typedef struct HXOrderRspInfo HXQueryOrderRsp;

/////////////////////////////////////////////////////////////////////////
///成交回报查询
/////////////////////////////////////////////////////////////////////////
///查询成交报告请求-根据执行编号查询（保留字段）
struct HXQueryReportByExecIdReq
{
    ///系统订单ID 用户无需填写, 在HX_OEMS系统中唯一
    uint64_t                order_no;
    ///成交执行编号
    char                    trade_no[HX_TRADE_NO_LEN];

	HXQueryReportByExecIdReq()
		: order_no(0)
	{
		memset(trade_no, 0, sizeof(trade_no));
	}
};

///查询成交回报请求-查询条件
struct HXQueryTraderReq
{
    ///证券代码，可以为空，如果为空，则默认查询时间段内的所有成交回报
    char                    symbol[HX_SYMBOL_LEN];
    ///开始时间，格式为YYYYMMDDHHMMSSsss，为0则默认当前交易日0点
    int64_t                 begin_time; 
    ///结束时间，格式为YYYYMMDDHHMMSSsss，为0则默认当前时间
    int64_t                 end_time;  

	HXQueryTraderReq()
		: begin_time(0)
		, end_time(0)
	{
		memset(symbol, 0, sizeof(symbol));
	}
};

///成交回报查询响应结构体
typedef struct HXTradeReportInfo  HXQueryTradeRsp;

/////////////////////////////////////////////////////////////////////////
///账户资金查询响应结构体
/////////////////////////////////////////////////////////////////////////
struct HXQueryAssetRsp
{
    ///客户号
	char client_id[HX_ACCOUNT_NAME_LEN];
    ///币种
    HX_CURRENCY_TYPE currency;
    ///期初余额
    double begin_amount;
    ///本交易日入金金额
    double deposit;
    ///本交易日出金金额
    double withdraw;
    ///当前余额
    double current_amount;
    ///总资产
    double total_amount;
    ///可用资金
    double available;
    ///可取资金
    double withdrawable;
    ///累计买入成交证券占用资金
    double total_buy_amount;
    ///累计买入成交交易费用
    double total_buy_fee;
    ///累计卖出成交证券所得资金
    double total_sell_amount;
    ///累计卖出成交交易费用
    double total_sell_fee;
    ///冻结的保证金
    double frozen_margin;
    ///资金资产
    double captial_asset;
    ///证券资产
    double security_asset;
    ///本交易日内的平仓盈亏/实现盈亏
    double close_profit;
    ///本交易日内交纳的手续费
    double commission;
    ///当前持仓盈亏
    double position_profit;
    ///当前浮动盈亏
    double float_profit;
    ///在途资金
    double in_transit;

	HXQueryAssetRsp()
		: currency(HX_CURRENCY_TYPE_UNKNOWN)
		, begin_amount(0.0)
		, deposit(0.0)
		, withdraw(0.0)
		, current_amount(0.0)
		, total_amount(0.0)
		, available(0.0)
		, withdrawable(0.0)
		, total_buy_amount(0.0)
		, total_buy_fee(0.0)
		, total_sell_amount(0.0)
		, total_sell_fee(0.0)
		, frozen_margin(0.0)
		, captial_asset(0.0)
		, security_asset(0.0)
		, close_profit(0.0)
		, commission(0.0)
		, position_profit(0.0)
		, float_profit(0.0)
		, in_transit(0.0)
	{
		memset(client_id, 0, sizeof(client_id));
	}
};

/////////////////////////////////////////////////////////////////////////
///查询股票持仓情况
/////////////////////////////////////////////////////////////////////////
struct HXQueryStkPositionRsp
{
	/////////////////////////////////////////////////////////////////////////
	///@brief HX_STOCK_HOLDER_TYPE是股东号类型
	/////////////////////////////////////////////////////////////////////////
    ///客户号
    char				client_id[HX_ACCOUNT_NAME_LEN];
    ///股东号
    char				stock_holder[HX_STOCK_HOLDER_LEN];
    ///证券代码
    char                symbol[HX_SYMBOL_LEN];
    ///证券名称
    char                symbol_name[HX_SYMBOL_NAME_LEN];
    ///交易市场
    HX_EXCHANGE_TYPE    exchange;
    ///期初数量
    int64_t             quantity_begin;
    ///期初冻结数量
    int64_t             quantity_begin_frozen;
    ///当日买入成交数量
    int64_t             quantity_buy_today;
    ///当日买入未成交数量
    int64_t             quantity_buy_frozen_today;
    ///当日卖出成交数量
    int64_t             quantity_sell_today;
    ///当日卖出未成交数量
    int64_t             quantity_sell_frozen_today;
    ///可用数量
    int64_t             quantity_available;
    ///持仓数量
    int64_t             quantity_current;
    ///最新价
    double              last_price;
    ///成本价
    double              cost_price;
    ///浮动盈亏
    double              float_profit;
	///清算速度，T+N（0表示T+0，1表示T+1...)
	double              tn;
	
	HXQueryStkPositionRsp()
		: exchange(HX_EXCHANGE_UNKNOWN)
		, quantity_begin(0)
		, quantity_begin_frozen(0)
		, quantity_buy_today(0)
		, quantity_buy_frozen_today(0)
		, quantity_sell_today(0)
		, quantity_sell_frozen_today(0)
		, quantity_available(0)
		, quantity_current(0)
		, last_price(0.0)
		, cost_price(0.0)
		, float_profit(0.0)
		, tn(0.0)
	{
		memset(client_id, 0, sizeof(client_id));
		memset(stock_holder, 0, sizeof(stock_holder));
		memset(symbol, 0, sizeof(symbol));
		memset(symbol_name, 0, sizeof(symbol_name));
	}
};

#pragma pack()
#endif