#include "hx_trader_api.h"
#include <iostream>
#include <string>
#include <string.h>
#include <thread>
#include <chrono>

class TraderEvents : public HxTraderSpi
{
public:
	void OnOrderChange(const HXOrderRspInfo *order_info, const HX_MESSAGE *error_info, uint64_t session_id)
	{
		if (error_info && error_info->code < 0)
		{
			printf("\tinsert order failed, error code:%d, error message:%s\n", error_info->code, error_info->message);
		}
		else
		{
			printf("\tfunction:%s,session_id:%llu,order_no:%llu,order_no_source:%llu,order_status:%d,symbol:%s,quantity:%llu,price:%.3f,quantity_traded:%llu\n", 
				__FUNCTION__, 
				session_id, 
				order_info->order_no,
				order_info->order_no_source,
				order_info->order_status, 
				order_info->symbol,
				order_info->quantity,
				order_info->price,
				order_info->quantity_traded);
		}

		///未报,已报,部分成交情况下撤单
		switch (order_info->order_status)
		{
			///已报 部分成交情况撤单
			case HX_ORDER_STATUS_TYPE_INIT:
			case HX_ORDER_STATUS_TYPE_READY:
			case HX_ORDER_STATUS_TYPE_PART_TRADED:
			{	
				uint64_t cancel_order_no = pTrader_->CancelOrder(order_info->order_no, session_id);
				if (cancel_order_no == 0)
				{
					///下单失败
					HX_MESSAGE* error_msg_st = pTrader_->GetApiLastError();
					printf("Order cancel failed,order_no:%llu,error code:%d,error message:%s\n",
							order_info->order_no_source, 
							error_msg_st->code, 
							error_msg_st->message);
				}
				else
				{
					printf("Order cancel successfully,order_no:%llu,cancel_order_no:%llu\n",order_info->order_no_source, cancel_order_no);
				}
				
				break;
			}
			///废单 全部成交 部成部撤 全部撤单 内部撤单情况下查询资金信息、持仓、成交、委托
			case HX_ORDER_STATUS_TYPE_INVALID:
			case HX_ORDER_STATUS_TYPE_ALL_TRADED:
			case HX_ORDER_STATUS_TYPE_PART_CANCELED:
			case HX_ORDER_STATUS_TYPE_ALL_CANCELED:
			case HX_ORDER_STATUS_TYPE_INTERNAL_CANCELED:
			{
				int req_id = request_id_++;
				int req_code = pTrader_->ReqAsset(session_id, req_id);

				HXQueryOrderReq query_order_req;
				query_order_req.begin_time = 0;
				query_order_req.end_time   = 0;
				req_id = request_id_++;
				req_code = pTrader_->ReqOrders(&query_order_req, session_id, req_id);

				req_id = request_id_++;
				req_code = pTrader_->ReqPosition("", session_id, req_id);

				HXQueryTraderReq query_trader_req;
				query_trader_req.begin_time = 0;
				query_trader_req.end_time   = 0;
				req_code = pTrader_->ReqTrades(&query_trader_req, session_id, req_id);

				break;
			}
			default:
			break;
		}
	}

	void OnTradeChange(const HXTradeReportInfo *trade_info, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,order_no:%llu,symbol:%s,quantity:%llu,price:%.3f,trade_amount:%.3f\n", 
			__FUNCTION__, 
			session_id, 
			trade_info->order_no, 
			trade_info->symbol,
			trade_info->quantity,
			trade_info->price,
			trade_info->trade_amount);
	}

	void OnCancelOrderError(const HXOrderCancelRspInfo *cancel_info, const HX_MESSAGE *error_info, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,order_no:%llu\n", 
			__FUNCTION__, 
			session_id, 
			cancel_info->order_no);
	}

	void OnQueryOrderRsp(const HXQueryOrderRsp *order_info, const HX_MESSAGE *error_info, int request_id, bool is_last, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,request_id:%d,is_last:%d,order_no:%llu,order_no_source:%llu,quantity_traded:%llu,quantity_left:%llu\n", 
			__FUNCTION__, 
			session_id, 
			request_id, 
			is_last,
			order_info->order_no, 
			order_info->order_no_source,
			order_info->quantity_traded,
			order_info->quantity_left);
		if (is_last)
		{
			printf("----------------------------------------------------\n");
		}		
	}

	void OnQueryTradeRsp(const HXQueryTradeRsp *trade_info, const HX_MESSAGE *error_info, int request_id, bool is_last, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,request_id:%d,is_last:%d,order_no:%llu,symbol:%s,exchange:%d,action:%d,quantity:%llu,price:%.3f,trade_amount:%.3f,trade_time:%llu\n", 
			__FUNCTION__, 
			session_id, 
			request_id, 
			is_last,
			trade_info->order_no, 
			trade_info->symbol,
			trade_info->exchange,
			trade_info->action,
			trade_info->quantity,
			trade_info->price,
			trade_info->trade_amount,
			trade_info->trade_time);
		if (is_last)
		{
			printf("----------------------------------------------------\n");
		}		
	}

	void OnQueryPositionRsp(const HXQueryStkPositionRsp *position, const HX_MESSAGE *error_info, int request_id, bool is_last, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,request_id:%d,is_last:%d,client_id:%s,symbol:%s,quantity_current:%llu,quantity_available:%llu\n", 
			__FUNCTION__, 
			session_id, 
			request_id, 
			is_last,
			position->client_id, 
			position->symbol,
			position->quantity_current,
			position->quantity_available);
		if (is_last)
		{
			printf("----------------------------------------------------\n");
		}		
	}

	void OnQueryAssetRsp(const HXQueryAssetRsp *asset, const HX_MESSAGE *error_info, int request_id, bool is_last, uint64_t session_id)
	{
		printf("\tfunction:%s,session_id:%llu,request_id:%d,is_last:%d,client_id:%s,current_amount:%.2f,total_amount:%.2f,available:%.3f,security_asset:%.3f\n", 
			__FUNCTION__, 
			session_id, 
			request_id, 
			is_last,
			asset->client_id, 
			asset->current_amount,
			asset->total_amount,
			asset->available,
			asset->security_asset);
		if (is_last)
		{
			printf("----------------------------------------------------\n");
		}
	}

	void SetTraderApi(HxTraderApi* pHxTraderApi)
	{
		pTrader_ = pHxTraderApi;
	}

private:
	HxTraderApi*					pTrader_;
	int								request_id_;
};


int main(int argc, char* argv[])
{
	//////////////////////////////////////////////////////////////////////////
	///设置交易参数
	std::string account_user	= "";
	std::string account_password= "";
	std::string trade_ip	    = "";
	int trade_port				= 0;
	std::string save_file_path	= "";
	int terminal_id				= 0;
	std::string license			= "";

	//////////////////////////////////////////////////////////////////////////
	///初始化api对象
	HxTraderApi* pTrader = HxTraderApi::CreateTraderApi(terminal_id, save_file_path.c_str(), HX_LOG_LEVEL_DEBUG);
	TraderEvents*  spi	 = new TraderEvents;
	spi->SetTraderApi(pTrader);
	
	pTrader->SubscribePublicTopic(HX_TERT_TYPE_QUICK);///设置公有流(订单响应、成交回报)重传方式模式, HX_TERT_TYPE_QUICK只传送登录后公有流
	pTrader->SetVersion("V1.00");///设置软件版本,由用户自定义
	pTrader->SetDeveloperKey(license.c_str());///设置listens信息,
	pTrader->RegisterSpi(spi);///注册事件类对象
	//////////////////////////////////////////////////////////////////////////
	///登录交易服务器
	uint64_t  session_id =  pTrader->Login(  trade_ip.c_str()
											, trade_port
											, account_user.c_str()
											, account_password.c_str()
											, HX_PROTOCOL_TCP);
	if (session_id == 0)
	{
		HX_MESSAGE* error_msg_st = pTrader->GetApiLastError();
		printf("Login %s:%d failed, error code:%d, error message:%s\n", trade_ip.c_str(), trade_port, error_msg_st->code, error_msg_st->message);	 	 
		return 0;
	}
	else
	{
		HX_MESSAGE* error_msg_st = pTrader->GetApiLastError();
		printf("Login %s:%d success, session_id:%llu\n", trade_ip.c_str(), trade_port, session_id);
	}
	
	//////////////////////////////////////////////////////////////////////////
	///下单
	HXOrderInsertInfo order_info;
	strcpy(order_info.symbol, "600300");						///证券代码 客户端请求不带空格,以'\0'结尾
	order_info.exchange = HX_EXCHANGE_SH;						///交易市场
	order_info.price = 3.37;									///价格
	order_info.quantity = 100;									///数量 (股票单位为股, 债券单位为张, 基金单位为份)
	order_info.price_type = HX_PRICE_LIMIT;						///报单价格类型
	order_info.action = HX_ACTION_TYPE_BUY;    					///买卖方向	   
	order_info.business_type = HX_BUSINESS_TYPE_STOCK;  		///业务类型

	uint64_t order_no = pTrader->InsertOrder(&order_info, session_id);	
	if (order_no == 0)
	{
		///下单失败
		HX_MESSAGE* error_msg_st = pTrader->GetApiLastError();
		printf("Order insert failed,error code:%d, error message:%s\n", error_msg_st->code, error_msg_st->message);
	}
	else
	{
		printf("Order insert success,order_no:%llu\n", order_no);
	}

	///让主线程不退出
	while (true)
	{
		std::this_thread::sleep_for(std::chrono::seconds(3600));
	}
	HxTraderApi::Release(pTrader);
	return 0;
}
